<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VP_Shipping_Rate_Method extends WC_Shipping_Method {

	//Create instance
	public function __construct( $instance_id = 0 ) {
		$this->id                 = 'vp_shipping_rate';
		$this->instance_id        = absint( $instance_id );
		$this->method_title       = _x( 'VP Shipping Rate', 'admin', 'vp-shipping-rate' );
		$this->method_description = __( 'Shipping rate based on various conditions. ', 'vp-shipping-rate' );
		$this->supports           = array(
			'shipping-zones',
			'instance-settings',
		);
		$this->init();

		//Save custom fields
		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
		add_filter('woocommerce_shipping_' . $this->id . '_instance_settings_values', array( $this, 'save_custom_options' ));

	}

	//Initialize settings.
	private function init() {
		$this->instance_form_fields     = include __DIR__ . '/settings-instance.php';
		$this->init_settings();
		$this->title                    = $this->get_option( 'title', $this->method_title );
		$this->tax_status               = $this->get_option( 'tax_status' );
	}

	//Calculate cost
	public function calculate_shipping( $package = array() ) {
		$rate = array(
			'id'      => $this->get_rate_id(),
			'label'   => $this->title,
			'cost'    => 0,
			'package' => $package,
			'meta_data' => array(
				'free_shipping_overwrite' => ($this->get_option( 'free_shipping_overwrite', 'no' ) == 'yes'),
			)
		);

		//Find out shipping cost
		$matched_prices = $this->get_matching_prices();

		//Get info if we need lowest or highest price if theres multiple matches
		$cost_logic = self::get_option('cost_logic', 'low');
		$cost = min($matched_prices);
		if($cost_logic != 'low') {
			$cost = max($matched_prices);
		}

		//Check if a free coupon is used and can overwrite the cost
		if($this->get_option('free_shipping_with_coupon', 'no') == 'yes' && VP_Shipping_Rate_Helpers::cart_has_free_shipping_coupon()) {
			$cost = 0;
		}

		//Allow plugins to customize
		$cost = apply_filters( 'vp_shipping_rate_cost', $cost, $package, $this);
		$rate['cost'] = $cost;

		//Set rate
		$this->add_rate( $rate );
	}

	public function get_matching_prices() {

		//Find out shipping cost
		$pricings = $this->get_option('pricing', array());
		$matched_prices = array();

		//Get cart details
		$cart_details = VP_Shipping_Rate_Conditions::get_cart_details('pricings');

		//Loop through each cost setup and see if theres a match
		foreach ($pricings as $pricing_id => $pricing) {

			//Get the price
			$price = $pricing['cost'];

			//Loop through each condition and see if its a match
			$condition_is_a_match = VP_Shipping_Rate_Conditions::match_conditions($pricings, $pricing_id, $cart_details);

			//If no match, skip to the next one
			if(!$condition_is_a_match) continue;

			//Make it an array, so if multiple prices are matched, we can later decide which one to use
			$matched_prices[] = $price;

		}

		//If we don't have any custom costs, just add the base cost
		if(!$matched_prices) {
			$matched_prices[] = $this->get_option('cost', 0);
		}

		return $matched_prices;
	}

	public function is_available( $package ) {

		//Find out shipping costs
		$matched_prices = $this->get_matching_prices();
		
		//If matched prices contains -1, return false, so the method is not available
		if(in_array(-1, $matched_prices)) {
			return false;
		}

		return true;
	}

	//Generate html for custom settings fields
	public function generate_vp_shipping_rate_settings_pricing_table_html( $key, $data) {
		return $this->render_custom_setting_html($key, $data);
	}

	//Generate html for custom settings fields
	public function render_custom_setting_html($key, $data) {
		$field_key = $this->get_field_key( $key );
		$defaults = array(
			'title' => '',
			'disabled' => false,
			'class' => '',
			'css' => '',
			'placeholder' => '',
			'type' => 'text',
			'desc_tip' => false,
			'description' => '',
			'custom_attributes' => array(),
			'options' => array()
		);
		$data = wp_parse_args( $data, $defaults );
		$template_name = str_replace('vp_shipping_rate_settings_', '', $data['type']);
		ob_start();
		include( dirname( __FILE__ ) . '/views/html-admin-'.str_replace('_', '-', $template_name).'.php' );
		return ob_get_clean();
	}

	//To save extra fields
	public function save_custom_options($options) {

		//Save pricing options
		$prices = array();
		if ( isset( $_POST['vp_shipping_rate_pricing'] ) ) {
			foreach ($_POST['vp_shipping_rate_pricing'] as $pricing_id => $pricing) {

				$cost = wc_clean($pricing['cost']);
				$cost = str_replace(',','.',$cost);
				$prices[$pricing_id] = array(
					'cost' => (float)$cost,
					'conditions' => array()
				);

				//If theres conditions to setup
				$conditions = (isset($pricing['conditions']) && count($pricing['conditions']) > 0);
				if($conditions) {
					$prices[$pricing_id]['logic'] = wc_clean($pricing['logic']);
					foreach ($pricing['conditions'] as $condition) {
						$condition_details = array(
							'category' => wc_clean($condition['category']),
							'comparison' => wc_clean($condition['comparison']),
							'value' => $condition[$condition['category']]
						);

						$prices[$pricing_id]['conditions'][] = $condition_details;
					}
				}

				$options['pricing'] = $prices;

			}
		}

		return $options;
	}

	public function has_payment_method_condition() {
		$has_payment_method_condition = false;
		$costs = $this->get_option('pricing', array());
		foreach ($costs as $cost_id => $cost) {

			//Check for conditions if needed
			foreach ($cost['conditions'] as $condition_id => $condition) {
				if($condition['category'] == 'payment_method') {
					$has_payment_method_condition = true;
					break;
				}
			}

		}
		return $has_payment_method_condition;
	}

}
